// Queue.java
//	Implementation of a bounded-capacity FIFO queue of integers.
//	
//	The queue is represented as an array.
//
// Copyright (c) 1992,1993,1995 The Regents of the University of California.
// All rights reserved.  See copyright.h for copyright notice and limitation 
// of liability and disclaimer of warranty provisions.

package nachos.kernel.threads;

import nachos.Debug;
import nachos.machine.NachosThread;
import nachos.machine.Interrupt;

/**
 * Bounded-capacity queue. 
 */
public class Queue {

  /** Maximum capacity of the queue. */
  int C;

  /** Index of the first element. */
  int first;

  /** Number of elements currently in the queue. */
  int n;

  /** Pointer to an array that holds the contents. */
  int[] q;


  /**
   * Initialize a list, empty to start with.
   * Elements can then be added to the list.
   *
   * @param c The capacity of the queue.
   */
  public Queue(int c) {

    Debug.ASSERT((c >= 1), "Queue constructor: Invalid capacity!");
  
    C = c;
    first = 0;
    n = 0;
    q = new int[C];   // allocate an array of integers.
  }

  /**
   * 	Insert an integer at the end of the queue.
   *    Abort on overflow (i.e., if the queue is full).
   *
   * @param value The value to insert
   */
  public void insert(int value) {
    Debug.ASSERT(n != C, "Queue.insert: can't insert in a full queue!");
    q[(first+n)%C] = value;
    n++;
  }

  /**
   * 	Remove an integer from the front of the queue, returning its value.
   *	Abort on underflow (i.e., if the queue is empty).
   */
  public int remove() {
    
    int val;
    
    Debug.ASSERT(n != 0, "Queue.remove: can't remove from an empty queue!");
    val = q[first];
    first = (first+1)%C;
    n--;
    return val;
  }
  
  /**
   * 	Return true iff the queue is full.
   */
  public boolean isFull() {
    return (n == C);
  }
  
  /**
   * 	Return true iff the queue is empty.
   */
  public boolean isEmpty() {
    return (n == 0);
  }
  
}
